%% Read input data
clearvars
clc
warning('off','all');

for gh = 1:7
        %% Start computation
        tic
        savim = 0;
        dfn = ['Processing particle - ', int2str(gh)];
        disp(dfn);
        % Read data directory
        dirs = cell(3, 1);
        disp('Choose binary data ...');
        fn = ['D:\Work\Paper 3\00_Draft\Final_7part\Particle',int2str(gh),'\00_Bin'];
        %dirs{1} = uigetdir('D:\Work\Paper 3\00_Draft\Final_7part\','Select the binary volume data directory');
        dirs{1} = fn;
        disp('Choose output directory...');
        %dirs{3} = uigetdir('D:\Work\Paper 3\00_Draft\Final_7part\','Select the output directory');
        fn = ['D:\Work\Paper 3\00_Draft\Final_7part\Particle',int2str(gh),'\01_Result_Output'];
        dirs{3} = fn;
        disp('Choose volume output directory ...');
        %dirs{4} = uigetdir('D:\Work\Paper 3\00_Draft\Final_7part\','Select the volume output folder');
        fn = ['D:\Work\Paper 3\00_Draft\Final_7part\Particle',int2str(gh),'\02_Volume_Output'];
        dirs{4} = fn;
        disp('Choose reactivity output directory ...');
        %dirs{5} = uigetdir('D:\Work\Paper 3\00_Draft\Final_7part\','Select the reactivity output folder');
        fn = ['D:\Work\Paper 3\00_Draft\Final_7part\Particle',int2str(gh),'\03_Reactivity_Output'];
        dirs{5} = fn;

        % Read data into matrix
        disp('1. Reading input data');
        kern = 1; % Choose size of windows kernel
        init_vol = single(seq2array(dirs{1},kern));
        init_vol(init_vol==2) = 255;

        %% Calculate gaussian curvature
        [Nb, ~, ~, ~]=NB3D(init_vol);
        tmp_Nb = zeros(size(init_vol));
        tmp_Nb(Nb(1).tot < 7 & Nb(1).tot ~= 0) = 1;
        % Recalculate gaussian curvature
        % Always re-check the gaussian smoothing inside the function !!!!
        [init_curvature] = calculateImageCurvature(init_vol,3); 
        init_curvature = init_curvature.*tmp_Nb;
        init_curvature(init_curvature<0) = 0;
        % Always re-check the gaussian smoothing inside the function !!!!
        init_curvature = round(init_curvature,2);
        unq_curvature = unique(init_curvature);
        unq_curvature(unq_curvature == 0) = [];

        %% Calculate distance map towards each curvature values
        disp('2. Computing distance map for all curvature values');
        % Recalculate distance and reactivity map
        [curva] = distance_map_nonpar(init_vol,init_curvature,unq_curvature);

        %% Compute the reactivity map from application curvature
        disp('4. Computing reactivity vs spectra relation'); 
        load('D:\Work\Paper 3\00_Draft\01_IndividualPart7\Galena_Constant_01.mat')
        c = 1;
        [ReactMap] = reactivity_map(const_b,curva,unq_curvature,init_vol);
        ReactMap = round(ReactMap,1);
        unq_React = unique(ReactMap);
        unq_React(unq_React == 0) = [];

        %% Dissolution simulation
        t = 0; dt = 1e-3; % this is in hour, remember to adjust dt according to the t_react

        % % Convert reactivity to time domain
        % disp('4. Computing time domain reactivity map'); 
        % fun_y = @(x) 0.5615*exp(0.2188*x); %% Obtained from Initialization_v0.m
        % 
        % % Saving initial volume and surface area
        % disp('5. Dissolution simulation preparation');
        % % Converting reactivity to dissolution rate based on react-spectra correlation
        % Time_ReactMap = fun_y(ReactMap);

        % Variable initalization for data plotting
        SA = zeros(1,1); Vol = zeros(1,1); a = 1;
        % [x_s,~,~] = findND(Nb(1).tot < 7 & Nb(1).tot ~= 0); % Checking surface area
        [x_c,~,~] = findND(init_vol == 1); % Checking volume
        % SA(a) = numel(x_s);
        Vol(a) = numel(x_c);
        [tt] = find(ReactMap > 0);
        RrMap(a,5) = SA_Calculation(init_vol);
        RrMap(a,4) = Vol(a);
        RrMap(a,3) = size(tt,1);
        RrMap(a,2) = sum(ReactMap,'all');
        RrMap(a,1) = t;
        count = Vol(1);

        % Display particle volume and curvature size
        disp('6. Simulation started');
        dd = ['Particle volume : ', num2str(count), ' voxel'];
        disp(dd);
        dd = ['Curvature size : ', num2str(numel(unq_curvature))];
        disp(dd);
        dd = ['Simulation time : ', num2str(t)];
        disp(dd);

        fn = ['\',num2str(c),'_ReactMapping_',num2str(t)];
        savim3D(dirs{5},fn,ReactMap);
        fn = ['\',num2str(c),'_InitVol_',num2str(t)];
        savim3D(dirs{4},fn,init_vol);

        while count > 64 % Stopping iteration when simulated volume was smaller than experiment volume
            t = t+dt;
            % Calculating the most reactive part from the surface
            [t_React] = parunq_react(unq_React);
            t_React(t_React == Inf) = [];
            t_React = round(t_React,2);

            % xt = t - t_React .* floor(t ./ t_React);
            % xt = ~mod(t,t_React) == 1; 
            % ax = find(xt == 1);
            xt = (mod(t,t_React)<1e-3) == 1; 
            ax = find(xt == 1);
            if numel(ax) > 0
                tmp_unq = unq_React(ax);
                for j=1:numel(tmp_unq)
                    [x,y,z] = findND(ReactMap == tmp_unq(j));
                    for k=1:numel(x)
                        if Nb(1).tot(x(k),y(k),z(k)) < 7
                            init_vol(x(k),y(k),z(k)) = 0;
                        end
                    end
                end
                init_vol(Nb(1).tot <= 2) = 0;

                a = a+1;
                %% Update volume condition everytime a voxels was removed
                [Nb, ~, ~, ~]=NB3D_efficient(init_vol); % Updating voxels that are exposed to solution
                tmp_Nb = zeros(size(init_vol));
                tmp_Nb(Nb(1).tot < 7 & Nb(1).tot ~= 0) = 1;
                % [x_s,~,~] = findND(Nb(1).tot < 7 & Nb(1).tot ~= 0); % Checking surface area
                [x_c,~,~] = findND(init_vol == 1); % Checking volume
                % SA(a) = numel(x_s); 
                Vol(a) = numel(x_c);
                count = Vol(a);
                
                % Recalculate gaussian curvature
                [init_curvature] = calculateImageCurvature(init_vol,3);
                init_curvature = init_curvature .* tmp_Nb; % Masking
                init_curvature(init_curvature<0) = 0;
                init_curvature = round(init_curvature,2);
                unq_curvature = unique(init_curvature);
                unq_curvature(unq_curvature == 0) = [];
                
                % Recalculate distance and reactivity map
                [curva] = distance_map_nonpar(init_vol,init_curvature,unq_curvature);
                [ReactMap] = reactivity_map(const_b,curva,unq_curvature,init_vol);
                [tt] = find(ReactMap > 0);
                RrMap(a,5) = SA_Calculation(init_vol);
                RrMap(a,4) = Vol(a);
                RrMap(a,3) = size(tt,1);
                RrMap(a,2) = sum(ReactMap,'all');
                RrMap(a,1) = t;
                ReactMap = round(ReactMap,1);
                unq_React = unique(ReactMap);
                % Time_ReactMap = fun_y(ReactMap);

                % Update volume and saving new evolution of reactivity map
                if mod(a,10) == 0 
                    % Showing volume
                    dd = ['Particle volume : ', num2str(count), ' voxel'];
                    disp(dd);
                    % Showing curvature size
                    dd = ['Curvature size : ', num2str(numel(unq_curvature))];
                    disp(dd);
                    dd = ['Simulation time : ', num2str(t)];
                    disp(dd);
                    % Saving updated reactivity map
                    c = c+1;
                    fn = ['\',num2str(c),'_ReactMapping_',num2str(t)];
                    savim3D(dirs{5},fn,ReactMap);
                    fn = ['\',num2str(c),'_InitVol_',num2str(t)];
                    savim3D(dirs{4},fn,init_vol);
                    % SA = SA'; 
                    Vol = Vol';
                    fn = [dirs{3},'\Output.mat'];
                    save(fn,"RrMap");
                end
            end
        end

        %% Saving variable
        Vol = Vol';
        fn = [dirs{3},'\Output_Simulation.mat'];
        save(fn,"RrMap");

        disp('7. Simulation finished');
        pn = ['Processing time : ', num2str(toc/3600), ' hours'];
        disp(pn);
end