function [gaussian_curvature] = calculateImageCurvature(image, kern)
    % Apply Gaussian smoothing to the input image
    image = imgaussfilt3(image, 0.7); %%%%%%%%%%%%%%%% CHECK THIS!!!!
    image(image > 1) = 0;
    % Compute the gradient structure tensor
    [Gx, Gy, Gz] = gradient(image);

    % Initialize cell array to store temporary results
    gaussian_curvature_cell = cell(numel(image), 1);

    % Loop over all voxels in the image
    parfor idx = 1:numel(image)
        [i, j, k] = ind2sub(size(image), idx);

        % Determine the bounds of the local neighborhood
        z_range = max(1, i - floor(kern / 2)):min(size(image, 1), i + floor(kern / 2));
        y_range = max(1, j - floor(kern / 2)):min(size(image, 2), j + floor(kern / 2));
        x_range = max(1, k - floor(kern / 2)):min(size(image, 3), k + floor(kern / 2));

        % Extract local gradient components within the neighborhood
        local_Gx = Gx(z_range, y_range, x_range);
        local_Gy = Gy(z_range, y_range, x_range);
        local_Gz = Gz(z_range, y_range, x_range);

        % Compute the local structure tensor elements
        Ixx = local_Gx.^2;
        Iyy = local_Gy.^2;
        Izz = local_Gz.^2;
        Ixy = local_Gx .* local_Gy;
        Ixz = local_Gx .* local_Gz;
        Iyz = local_Gy .* local_Gz;

        % Construct the local structure tensor
        local_S = [sum(Ixx(:)) sum(Ixy(:)) sum(Ixz(:));
                   sum(Ixy(:)) sum(Iyy(:)) sum(Iyz(:));
                   sum(Ixz(:)) sum(Iyz(:)) sum(Izz(:))];

        % Solve the eigenvalue problem for the local structure tensor
        [~, D] = eig(local_S);

        % Extract principal curvatures from the eigenvalues
        principal_curvatures = diag(D);

        % Calculate Gaussian curvature as the product of principal curvatures
        gaussian_curvature_cell{idx} = principal_curvatures(1) * principal_curvatures(2);
    end

    % Reconstruct the matrix from the cell array
    gaussian_curvature = reshape(cell2mat(gaussian_curvature_cell), size(image));
end