function [curvaBlocks] = distance_map_par(blocks_vol, blocks_curvature, unq_curvature)
    % Initialize the cell array to store the processed blocks
    curvaBlocks = cell(size(blocks_vol));
    I = size(blocks_vol, 1);
    J = size(blocks_vol, 2);
    K = size(blocks_vol, 3);
    % Process each block using parallel computation
    parfor i = 1:I
        for j = 1:J
            for k = 1:K
                if k ~= K
                    % Extract coordinates of non-zero values in the block
                    [xv, yv, zv] = findND(blocks_vol{i, j, k} == 1);
                    xv = single(xv); yv = single(yv); zv = single(zv);
                    
                    % Initialize struct for current curvature
                    curva = struct('init_dist', zeros(size(blocks_vol{i, j, k}), 'single'));
                    
                    % Process unique curvatures for the block
                    for c = 1:numel(unq_curvature)
                        [xc, yc, zc] = findND(blocks_curvature{i, j, k} == unq_curvature(c));
                        x1 = xv; y1 = yv; z1 = zv;
                        for m = 1:numel(xv)
                            x2 = xc; y2 = yc; z2 = zc;
                            d_sigma = 1e2;
                            for n = 1:numel(xc)
                                dx = x1(m) - x2(n);
                                dy = y1(m) - y2(n);
                                dz = z1(m) - z2(n);
                                d_tmp = sqrt(dx^2 + dy^2 + dz^2);
                                if d_tmp < d_sigma
                                    d_sigma = d_tmp;
                                end
                            end
                            curva.init_dist(xv(m), yv(m), zv(m)) = d_sigma;
                        end
                    end
                    % Store the processed block in the cell array
                    curvaBlocks{i, j, k} = curva;
                end
            end
        end
    end
end